@@ -32,7 +32,7 @@
 ## $Id: $
 ##
 
-create3D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
+create3D <- function(dcm, mode="integer", transpose=FALSE, pixelData=TRUE,
                      mosaic=FALSE, mosaicXY=NULL, sequence=FALSE) {
   if (pixelData) {
     if (is.null(dcm$hdr)) {
@@ -68,13 +68,13 @@ create3D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
       x <- mosaicXY[1]
       y <- mosaicXY[2]
     }
-    z <- (X/x) * (Y/y)
+    z <- (X / x) * (Y / y)
     img <- array(0, c(x,y,z))
     k <- 1
-    for (i in (X/x):1) {
-      for (j in 1:(Y/y)) {
-        img[,,k] <- dcm$img[((i-1)*x)+1:x, ((j-1)*y)+1:y]
-        k <- k+1
+    for (j in (Y/y):1) {
+      for (i in 1:(X/x)) {
+        img[,,k] <- dcm$img[((i-1) * x) + 1:x, ((j-1) * y) + 1:y]
+        k <- k + 1
       }
     }
     storage.mode(img) <- mode
@@ -103,8 +103,8 @@ create3D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
       }
     } else {
       for (z in 1:Z) {
-        ## img[,,z] <- dicomInfo(names(dcm$hdr)[iop.order[z]])$img
-        img[,,z] <- dicomInfo(names(dcm$hdr)[z])$img
+        ## img[,,z] <- readDICOMFile(names(dcm$hdr)[iop.order[z]])$img
+        img[,,z] <- readDICOMFile(names(dcm$hdr)[z])$img
       }
     }
   }
@@ -117,7 +117,7 @@ create3D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
   return(img)
 }
 
-create4D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
+create4D <- function(dcm, mode="integer", transpose=FALSE, pixelData=TRUE,
                      mosaic=FALSE, mosaicXY=NULL, nslices=NULL,
                      ntimes=NULL, instance=TRUE, sequence=FALSE) {
   if (pixelData) {
@@ -185,7 +185,7 @@ create4D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
     } else {
       for (w in 1:ntimes) {
         k <- 1
-        dicomInfoImage <- dicomInfo(names(dcm$hdr)[w])$img
+        dicomInfoImage <- readDICOMFile(names(dcm$hdr)[w])$img
         for (i in (X/x):1) {
           for (j in 1:(Y/y)) {
             img[,,k,w] <- dicomInfoImage[((i-1)*x)+1:x, ((j-1)*y)+1:y]
@@ -237,7 +237,7 @@ create4D <- function(dcm, mode="integer", transpose=TRUE, pixelData=TRUE,
       for (z in 1:Z) {
         zz <- (z - 1) %% nslices + 1
         ww <- (z - 1) %/% nslices + 1
-        img[,,zz,ww] <- dicomInfo(names(dcm$hdr)[index[z]])$img
+        img[,,zz,ww] <- readDICOMFile(names(dcm$hdr)[index[z]])$img
       }
     }
   }
